#!/usr/bin/python
# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

import logging

from redis import Redis, Sentinel


class Config:
    pass


logger = logging.getLogger("remotecv")  # pylint: disable=invalid-name
config = Config()  # pylint: disable=invalid-name

SINGLE_NODE = "single_node"
SENTINEL = "sentinel"


def redis_client():
    if config.redis_mode == SINGLE_NODE:
        return __redis_single_node_client()

    if config.redis_mode == SENTINEL:
        return __redis_sentinel_client()

    raise AttributeError(f"redis-mode must be either {SINGLE_NODE} or {SENTINEL}")


def __redis_sentinel_client():
    instances_split = config.redis_sentinel_instances.split(",")
    instances = [instance.split(":") for instance in instances_split]

    if config.redis_sentinel_password:
        sentinel_instance = Sentinel(
            instances,
            socket_timeout=config.redis_sentinel_socket_timeout,
            sentinel_kwargs={"password": config.redis_sentinel_password},
        )
    else:
        sentinel_instance = Sentinel(
            instances,
            socket_timeout=config.redis_sentinel_socket_timeout,
        )

    return sentinel_instance.master_for(
        config.redis_sentinel_master_instance,
        socket_timeout=config.redis_sentinel_socket_timeout,
        password=config.redis_sentinel_master_password,
        db=config.redis_sentinel_master_database,
    )


def __redis_single_node_client():
    if config.redis_password:
        return Redis(
            host=config.redis_host,
            port=config.redis_port,
            db=config.redis_database,
            password=config.redis_password,
        )

    return Redis(
        host=config.redis_host,
        port=config.redis_port,
        db=config.redis_database,
    )



from remotecv.result_store import BaseStore
from remotecv.utils import logger, redis_client


class ResultStore(BaseStore):

    WEEK = 604800
    redis_instance = None

    def __init__(self, config):
        super().__init__(config)

        if not ResultStore.redis_instance:
            ResultStore.redis_instance = redis_client()
        self.storage = ResultStore.redis_instance

    def store(self, key, points):
        result = self.serialize(points)
        logger.debug("Points found: %s", result)
        redis_key = f"thumbor-detector-{key}"
        self.storage.setex(
            name=redis_key,
            value=result,
            time=2 * self.WEEK,
        )



from pathlib import Path

from setuptools import setup


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")

with open("remotecv/__init__.py") as f:
    ns = {}
    exec(f.read(), ns)
    version = ns["__version__"]


TESTS_REQUIREMENTS = [
    "celery==5.*,>=5.2.7",
    "flake8==4.*,>=4.0.1",
    "pylint==2.*,>=2.13.8",
    "black==22.*,>=22.1.0",
    "preggy==1.*,>=1.4.4",
    "pyssim==0.*,>=0.4",
    "pytest==7.*,>=7.0.0",
    "pytest-cov==3.*,>=3.0.0",
    "pytest-asyncio==0.*,>=0.18.0",
    "coverage==6.*,>=6.3.2",
    "thumbor==7.*",
]

RUNTIME_REQUIREMENTS = [
    "opencv-python-headless==4.*,>=4.2.0",
    "Pillow>=9.0.0",
    "pyres==1.*,>=1.5.0",
    "sentry-sdk==0.*,>=0.14.2",
    "redis==4.*,>=4.2.0",
]

setup(
    long_description=long_description,
    name="remotecv",
    version=version,
    description="remotecv is an OpenCV worker for facial and feature recognition",
    python_requires="==3.*,>=3.7.0",
    author="Bernardo Heynemann",
    author_email="heynemann@gmail.com",
    license="MIT",
    entry_points={"console_scripts": ["remotecv = remotecv.worker:main"]},
    classifiers=[
            "Development Status :: 4 - Beta",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Natural Language :: English",
            "Operating System :: MacOS",
            "Operating System :: POSIX :: Linux",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: 3.11",
            "Programming Language :: Python :: 3 :: Only",
            "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
            "Topic :: Multimedia :: Graphics :: Presentation",
    ],
    packages=[
        "remotecv",
        "remotecv.detectors",
        "remotecv.detectors.complete_detector",
        "remotecv.detectors.face_detector",
        "remotecv.detectors.feature_detector",
        "remotecv.detectors.glasses_detector",
        "remotecv.detectors.profile_detector",
        "remotecv.result_store",
    ],
    package_dir={"": "."},
    package_data={
        "remotecv.detectors.face_detector": ["*.xml"],
        "remotecv.detectors.glasses_detector": ["*.xml"],
        "remotecv.detectors.profile_detector": ["*.xml"],
    },
    install_requires=[RUNTIME_REQUIREMENTS],
    extras_require={"dev": [RUNTIME_REQUIREMENTS + TESTS_REQUIREMENTS]},
)
